# -*- coding: utf-8 -*-
# python methods for the system module
from global_defines import * 
import system
import math
import helpfile

__doc__ = """Add-in module for module system.

This module isn't meant to be used directly. Everything defined here will be
part of the system module.
"""

def register_exit( regionname ):
	"""Registers the regions where the player chars can leave the zone.

Example:
        system.add_region(id="REGION_EXIT_NORD",  x1=1000, y1=800, x2=810, y2=0)
        system.register_exit(regionname="REGION_EXIT_NORD")
"""
	if not hasattr( data(), "registered_exit_zones" ):
		data().registered_exit_zones = []
	data().registered_exit_zones.append( regionname )

def timed_callback(delay, callback):
	"""Execute callback after delay

Parameters:
        delay - time in ms until the callback is executed
        callback - callback to execute
"""
	import random
	
	if not hasattr(data("timed_callback"), "callbacks"): data("timed_callback").callbacks = {}
	
	callback_id = str(random.random())
	while callback_id in data("timed_callback").callbacks: callback_id = str(random.random())
	
	data("timed_callback").callbacks[callback_id] = callback
	
	system.notify_global_timer("timed_callback_" + callback_id, delay)

ZONE_PATH = 0
ZONE_COORD = 1

zones_list = {
	"zone_1":	("zones\\zone_1\\zone_1", [391.00, 504.00]),
	"zone_2":	("zones\\zone_2\\zone_2", [378.00, 429.50]),
	"zone_3":	("zones\\zone_3\\zone_3", [317.00, 338.00]),
	"zone_4":	("zones\\zone_4\\zone_4", [288.00, 436.00]),
	"zone_5":	("zones\\zone_5\\zone_5", [120.00, 390.50]),
	"zone_6":	("zones\\zone_6\\zone_6", [164.00, 204.00]),
	"zone_7":	("zones\\zone_7\\zone_7", [357.00, 126.00]),
	"zone_8":	("zones\\zone_8\\zone_8", [411.00, 205.50]),
	"zone_9":	("zones\\zone_9\\zone_9", [590.00, 240.50]),
	"zone_10":	("zones\\zone_10\\zone_10", [610.50, 425.00]),
	
	"zzone_1":	("zones\\zzone_1\\zzone_1", [219.00, 389.50]),
	"zzone_2":	("zones\\zzone_2\\zzone_2", [130.00, 483.00]),
	"zzone_3":	("zones\\zzone_3\\zzone_3", [135.00, 278.00]),
	"zzone_4":	("zones\\zzone_4\\zzone_4", [270.00, 169.00]),
	"zzone_5":	("zones\\zzone_5\\zzone_5", [309.00, 232.00]),
	"zzone_6":	("zones\\zzone_6\\zzone_6", [561.00, 164.50]),
	"zzone_7":	("zones\\zzone_7\\zzone_7", [513.00, 220.50]),
	"zzone_8":	("zones\\zzone_8\\zzone_8", [597.00, 328.50]),
	"zzone_9":	("zones\\zzone_9\\zzone_9", [71.00, 312.50]),
	"zzone_10":	("zones\\zzone_10\\zzone_10", [70.00, 312.50]),

	"sqzone_1":	("zones\\sqzone_1\\sqzone_1", [459.00, 375.00]),
	"sqzone_2":	("zones\\sqzone_2\\sqzone_2", [236.00, 288.50]),
	"sqzone_3":	("zones\\sqzone_3\\sqzone_3", [ 55.50, 447.50]),
	"sqzone_5":	("zones\\sqzone_5\\sqzone_5", [236.00, 481.50]),
	"sqzone_6":	("zones\\sqzone_6\\sqzone_6", [353.00, 273.00]),
}

def get_zone_data(zone, data=None):
	"""Returns information about the zone.

Parameters:
        zone - Short id of the zone (e.g. "zone_5").
               Please note that nearly everywhere else the long id 
               ("zone_5.zone_5") is used.
        data - States what data to return:
                ZONE_PATH  - Zone path (e.g. "zones\zzone_5\zzone_5").
                ZONE_COORD - Zone coordinates on the travel map
                             (e.g. [309.00, 232.00]).
                None       - Returns both information
                (e.g. ("zones\zzone_5\zzone_5", [309.00, 232.00])).
"""
	if zones_list.has_key(zone):
		if data==None:
			return zones_list[zone]
		else:
			return zones_list[zone][data]
	else:
		raise ValueError, "no zone with this identifier registered '" + zone + "'" 

def get_travel_time(start_zone_id, target_zone_id, vehicle_list):
	"""Returns the travel time (in ms).

Parameters:
        start_zone_id  - Short zone id (e.g. "zone_5").
        target_zone_id - Short zone id (e.g. "zone_6").
        vehicle_list   - List of vehicle the player party has.
                         Travel is faster with vehicles.
"""
	p1 = get_zone_data(start_zone_id, ZONE_COORD)
	p2 = get_zone_data(target_zone_id, ZONE_COORD)
	dx = p1[0]-p2[0]
	dy = p1[1]-p2[1]
	distance = math.sqrt((dx*dx) + (dy*dy))
	#print "distance", start_zone_id, target_zone_id, distance
	time = 0
	per_hour = 5
	if vehicle_list!=None:
		if len(vehicle_list)>0:
			per_hour = 25
	hours = distance/per_hour
	days = hours/15
	time = (hours*60*60*1000)+(days*24*60*60*1000)
	return time

def get_enter_direction(start_zone_id, target_zone_id):
	"""Calculates the enter direction for the new zone from the position of both
zones on the travel map.

Parameters:
        start_zone_id  - Short zone id (e.g. "zone_5").
        target_zone_id - Short zone id (e.g. "zone_5").
"""
	p1 = get_zone_data(start_zone_id, ZONE_COORD)
	p2 = get_zone_data(target_zone_id, ZONE_COORD)
	X = 0
	Y = 1
	result = None
	if p1[X]>p2[X]: 
		if p2[Y]-p1[Y] > 50:
			result = "NORTH"
		elif p2[Y]-p1[Y] < -50:
			result = "SOUTH"
		else:
			result = "EAST" 
	else:	 
		if p2[Y]-p1[Y] > 50:
			result = "NORTH"
		elif p2[Y]-p1[Y] < -50:
			result = "SOUTH"
		else:
			result = "WEST"
	return result

def get_registered_zones():
	"""Returns a list with all possible zones.

Returns a list of the short ids of the zones.
The list entries can be used as parameters for get_zone_data and similar functions.
"""
	global zones_list
	return zones_list.keys()

def get_region_cursor( regionname ):
	"""Allows setting a special mouse cursor depending on the region the
mouse is over.

Also see system.add_region.

Parameters:
        regionname - Id of the region(as set with add_region).
"""
	CURSOR_ZONE_EXIT = 16
	if 	hasattr(data(), "registered_exit_zones") and \
		regionname in data().registered_exit_zones:
		return CURSOR_ZONE_EXIT
	else:
		return -1

def can_trigger_region(id, objectid, partyid):
	"""Returns True when the region can be triggered.

This allows for disabling the region system, as is done while resting.

Parameters:
        id       - Id of the triggered region.
        objectid - Id of the entering object.
        partyid  - Id of the party of entering object.
"""
	# don't trigger regions when resting
	if hasattr(data("rest"), "raid") and data("rest").raid:
		return False
	if hasattr(data("rest"), "resting") and data("rest").resting:
		return False

	return True

#derff
def backpack_clear():
	"""Empty all containers when leave zone"""
	helpfile.itemers_check()
	for pc in system.get_pcs():
		inventory = objects.get_inventory_id_list(pc)
		for item in inventory: 
			if objects.has_attribute(item,"container"):
				inv = objects.get_inventory_id_list(item)
				itemer = helpfile.create_itemer(item)
				for i in inv:
					objects.transfer_item(item, i, itemer)
				objects.set_attribute(item,"itemer",itemer)
				objects.set_attribute(itemer, "zone_transfer", True )
				objects.set_attribute(itemer, "zone_transfer_new_party", "NPCS" )
				data("ALTER_EGO").itemers.append(itemer)
				objects.set_active_state(itemer,1)
	print "Ready to zonetranfer containers", data("ALTER_EGO").itemers

def get_combo_item_list():
	"""
	Returned list for items with combinations for get_combo_items. It save this list in memory, so it's no
	need to seach full items list for each combo.
"""
	if not hasattr ( data(), "combo_item_list" ) or not hasattr ( data(), "combo_item_list_version" ) \
		or data().combo_item_list_version < 1 :
		combo_list = []
		item_list = system.get_item_list()
		for item in item_list:
			if objects.has_attribute ( item, "item_combination" ):
				combo_list.append ( item )
		data().combo_item_list = combo_list
		data().combo_item_list_version = 1
	return data().combo_item_list

# add methods to object module
import sys
sys.modules["system"].register_exit = register_exit
sys.modules["system"].timed_callback = timed_callback
sys.modules["system"].get_region_cursor = get_region_cursor
sys.modules["system"].get_zone_data = get_zone_data
sys.modules["system"].get_travel_time = get_travel_time
sys.modules["system"].get_enter_direction = get_enter_direction
sys.modules["system"].get_registered_zones = get_registered_zones
sys.modules["system"].can_trigger_region = can_trigger_region
sys.modules["system"].ZONE_PATH = ZONE_PATH
sys.modules["system"].ZONE_COORD = ZONE_COORD
sys.modules["system"].zones_list = zones_list
sys.modules["system"].backpack_clear = backpack_clear
sys.modules["system"].get_combo_item_list = get_combo_item_list

